import logging
import sys
import traceback

from vlcishared.mail.mail import Email

FAILED_EXEC = 1
SUCCESS_EXEC = 0


class FlowControl:
    '''Clase que implementa métodos que se usan para controlar
    el flujo de la ETL, por ejemplo para terminar la ejecución,
    para permitir que continue pero que envie un correo al final
    o para indicar que la ejecución ha sido exitosa'''

    flow_state = None

    def __init__(self, mail: Email = None) -> None:
        self.flow_state = SUCCESS_EXEC
        self.log = logging.getLogger()
        self.mail = mail

    def handle_error(self, cause: str, fatal: bool = False) -> None:
        '''Logea el error recibido, lo añade al correo que se va a enviar y
        si es un error fatal termina la ejecución de la ETL'''
        self.flow_state = FAILED_EXEC
        self.log.error(cause)
        self.log.error(f'Excepción: {traceback.format_exc(limit=1)}')

        if self.mail is not None:
            self.mail.append_line('Ejecución Fallida: ETL KO.')
            self.mail.append_line(f'Causa del fallo: {cause}')

        if fatal:
            self.end_exec()

    def end_exec(self):
        '''Termina la ejecución de la ETL retornando 1 en caso de fallo y 0
        en caso se exito'''
        if self.flow_state == FAILED_EXEC:
            self.log.info('Ejecución Fallida, ETL KO')
            if self.mail is not None:
                self.mail.append_line('Se adjuntan logs de la ejecución.')
                self.mail.add_attachment(
                    self.log.handlers[0].baseFilename, 'text/plain')
                self.mail.send()
        else:
            self.log.info('Ejecución Exitosa, ETL OK')

        return sys.exit(self.flow_state)



import pysftp

from vlcishared.utils.interfaces import ConnectionInterface


class SFTPClient(ConnectionInterface):
    '''Clase que se conecta a un servidor sftp para descargar ficheros'''

    def __init__(self, host: str, username: str, password: str, port=22):
        '''Necesita el host, username, el password y el puerto al que se va a
            conectar para inicializar el cliente'''
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.cnopts = pysftp.CnOpts()
        self.cnopts.hostkeys = None

    def connect(self) -> None:
        '''Intenta conectarse al servidor configurado en la instanciación'''
        try:
            self.sftp = pysftp.Connection(
                host=self.host,
                port=self.port,
                username=self.username,
                password=self.password,
                cnopts=self.cnopts
            )
        except Exception as e:
            raise ConnectionRefusedError(f"Conexión fallida: {str(e)}")

    def list(self, remote_path: str) -> list:
        '''Devuelve una lista con los ficheros en el directorio indicado'''
        try:
            with self.sftp.cd(remote_path):
                remote_files = self.sftp.listdir()
            return remote_files
        except Exception as e:
            raise ConnectionAbortedError(
                f"Fallo al listar los archivos: {str(e)}")

    def download(self, remote_path: str, local_path: str, file_name: str):
        '''Descarga el fichero recibido como parámetro y
            lo guarda en el "local_path"'''
        try:
            self.sftp.get(f'{remote_path}/{file_name}',
                          f'{local_path}/{file_name}')
        except Exception as e:
            raise ConnectionAbortedError(f"Descarga fallida: {str(e)}")

    def move(self,
             remote_origin_path: str,
             destiny_path: str,
             file_name: str):
        '''Mueve el fichero recibido como parámetro de
        la carpeta origen a la carpeta destino'''
        try:
            self.sftp.rename(f'{remote_origin_path}/{file_name}',
                             f'{destiny_path}/{file_name}')
        except Exception as e:
            raise ConnectionAbortedError(
                f"Fallo moviendo el fichero de "
                f"{remote_origin_path} a {destiny_path}: {str(e)}")

    def close(self):
        '''Cierra la conexión al servidor SFTP'''
        self.sftp.close()
        print(f"Conexión a {self.host} cerrada.")



import logging
from typing import Optional

from vlcishared.db.postgresql import PostgresConnector


class GestionFechas:

    def __init__(self, db_connector: PostgresConnector):
        self.db_connector = db_connector
        self.log = logging.getLogger()

    def gestion_fechas_inicio(
            self,
            identifier: str,
            param: str
    ) -> Optional[str]:
        """
        Gestión de fechas inicio: Se averigua la fecha, se obtiene y
        se registra el estado a 'En Proceso'
        """
        try:
            result = self.db_connector.call_procedure(
                'p_gf_averiguarejecucion',
                identifier,
                is_function=True
            )
            if result[0][1] == "f":
                return result[0]
            else:
                date_result = self.db_connector.call_procedure(
                    'p_gf_obtenerfecha',
                    identifier,
                    param,
                    is_function=True
                )
                self.db_connector.call_procedure(
                    'p_gf_registrarestadoetl',
                    identifier,
                    'EN PROCESO',
                    is_function=True
                )
                self.log.info("Gestión de fechas inicio completada.")
                return date_result[0][0]
        except Exception as e:
            self.log.error(f"Error en gestión de fechas inicio: {e}")
            raise e

    def gestion_fechas_fin(
            self,
            identifier: str,
            param: str,
            status: str
    ) -> Optional[str]:
        """Gestión de fechas fin: Se registra el estado a 'OK' o 'ERROR',
            se calcula la nueva fecha y se actualiza"""
        try:
            self.db_connector.call_procedure(
                'p_gf_registrarestadoetl',
                identifier,
                status,
                is_function=True
            )
            result = self.db_connector.call_procedure(
                'p_gf_calcularNuevaFecha',
                identifier,
                is_function=True
            )

            self.log.info("Gestión de fechas fin completada.")
            return result[0][0]
        except Exception as e:
            self.log.error(f"Error en gestión de fechas fin: {e}")
            raise e
